
#include "command_processors.h"

int soft_reset(char* fields[]) {
    if ((*fields[1] == '0') || (*fields[1] == 0)) {
        printf("Resetting...\n\r");
        tb_delay_ms(1000);
        printf("\033[2J"); // clear screen
        tb_delay_ms(100);
        __asm__ volatile ("reset");
    }     return 0;
}

int sleep(char* fields[]) {
    if ((*fields[1] == '0') || (*fields[1] == 0)) {
        __asm__ volatile ("pwrsav #0");
    } 
    return 0;
}

void dout(int do_num, dout_modes mode, int on, int off, int invert) {
    switch (mode) {
        case PULSE:
            pwm_timer_set(do_num, on, 0, do_num, invert);
            break;
        case PWM:
            pwm_timer_set(do_num, on, off, do_num, invert);
            break;
        case DO_ON:
            switch (do_num) {
                case 1:
                    DOUT1 = 1;
                    break;
                case 2:
                    DOUT2 = 1;
                    break;
                case 3:
                    DOUT3 = 1;
                    break;
                case 4:
                    DOUT4 = 1;
                    break;
                case 5:
                    DOUT5 = 1;
                    break;
            }
            break;
        case DO_OFF:
            switch (do_num) {
                case 1:
                    DOUT1 = 0;
                    break;
                case 2:
                    DOUT2 = 0;
                    break;
                case 3:
                    DOUT3 = 0;
                    break;
                case 4:
                    DOUT4 = 0;
                    break;
                case 5:
                    DOUT5 = 0;
                    break;
            }
            break;
    }
}

int vtim(char* fields[]) {
    if ((*fields[1] == '0') || (*fields[1] == 0)) {
        long V;
        char voltage[6]; // buffer to hold results string
        V = ((long) (adc_get())/2);
        sprintf(voltage, "%ld", V);
        printf("%s\n\r", voltage);
    }
    return 0;
}

int led_test(char* fields[]) {
    if ((*fields[1] == '0') || (*fields[1] == 0)) {
        pwm_timer_set(3, 0, 0, LED1, NOINVERT);
    } else if (*fields[1] == '1') {
        pwm_timer_set(3, 250, 250, LED1, NOINVERT);
    }
    return 0;
}

int dout_test(char* fields[]) {
    if ((*fields[1] == '0') || (*fields[1] == 0)) {
        pwm_timer_set(4, 0, 0, DO1, NOINVERT);
    } else if (*fields[1] == '1') {
        pwm_timer_set(4, 1000, 1000, DO1, NOINVERT);
    }
    return 0;
}

int switch_test(char* fields[]) {
    printf("Press switch to activate OC output.\n\r");
    pwm_timer_set(6, 10000, 0, 0, 0);
    while (!pwm_timer(6)); // wait until timer started
    while (pwm_timer(6)) DOUT1 = ~PORTBbits.RB4; // led when switch pressed
    DOUT1 = 0; // test timer finished, turn off output
    printf("Switch test completed.\n\r");
    return 0;
}

void show_err(int err_num) {
    switch (err_num) {
        case 1:
            printf("Invalid input.\n\r");
            break;
        case 2:
            printf("Unrecognized command.\n\r");
            break;
        default:
            break;
    }
}

unsigned int read_dipsw(void) {
    unsigned int probe = 1;
    unsigned int result = 0;
    int i;
    for (i = 0; i < 4; i++) { //probe each switch to determine state
        if (probe & 0b1000) DIPSW4 = 0; //if switch 4 is open, it's a 0
        else DIPSW4 = 1; // if closed, it's a 1
        if (probe & 0b0100) DIPSW3 = 0;
        else DIPSW3 = 1;
        if (probe & 0b0010) DIPSW2 = 0;
        else DIPSW2 = 1;
        if (probe & 0b0001) DIPSW1 = 0;
        else DIPSW1 = 1;
        result = (result << 1) | SENSE;
        probe = probe << 1;
        DIPSW4 = 1; // leave output HI on all lines to minimize current draw
        DIPSW3 = 1;
        DIPSW2 = 1;
        DIPSW1 = 1;
    }
    return ~result & 0x0f;
}

int dipsw(char* fields[]) {
    printf("%x\n\r", read_dipsw());
    return 0;
}
