/* 
 * File:   command_processors.h
 * Author: WHalphen
 *
 * Created on October 20,2016
 * 
 * Command Processors
 * Collection of functions to process commands
 * 
 * void soft_reset(void);
 *    purpose: does a software reset of the processor, causing it to restart
 *    its program
 * void sleep(void);
 *    purpose: shuts down the processor - requires a hardware reset to start up
 *    again
 * void show_err(int err_num);
 *    purpose: displays a message for the specified error code
 *    parameter: err_num is the error code to be reported
 * void dout(int do_num, dout_modes mode, int on, int off, int invert);
 *    purpose: controls a digital output by turning it on, off, sending a pulse,
 *    or sending a PWM series to a specified output
 *    parameters: 
 *       do_num - index of the digital output to be used
 *       mode - indicates output mode: PULSE, PWM, DO_ON, DO_OFF
 *          PULSE outputs a times one-shot pules
 *          PWM outputs a timed pulse width modulated ongoing sequence
 *          DO_ON turns an output on (high)
 *          DO_OFF turns an output off (low)
 *       on - number of milliseconds for the output to be on (PULSE and PWM modes)
 *       off - number of milliseconds for the output to be off (PWM mode)
 *       invert - if 1, the output is inverted (PULSE and PWM modes)
 *          if 0, the output is not inverted 
 */

#ifndef COMMAND_PROCESSORS_H
#define	COMMAND_PROCESSORS_H

#include <stdio.h>
#include <stdlib.h>
#include "msg_processor.h"
#include "timebase.h"
#include "adc.h"

int soft_reset(char* fields[]);
int sleep(char* fields[]);
void show_err(int err_num);
void dout(int do_num, dout_modes mode, int on, int off, int invert);

void show_err(int err_num);
int led_test(char* fields[]);
int dout_test(char* fields[]);
int switch_test(char* fields[]);
int vtim(char* fields[]);
unsigned int read_dipsw(void); //probes the dip switches to get their states
int dipsw(char* fields[]);

#endif	/* COMMAND_PROCESSORS_H */

