/* 
 * File:   main.c
 * Author: WHalphen
 *
 * Created on December 17, 2015, 9:36 AM
 */

#include "main.h"

long V; // variable to hold voltage measurement from timer potentiometer
char milliseconds[6]; // buffer to hold results string

void oneshot(int mult) {
    if (AUTO_MAN == AUTO) {
        if (trigger_status()) { // if trigger switch pressed
            printf("auto trigger: ");
            trigger_clear(); // clear status
            V = (long) ((adc_get()) * mult) / 10;
            sprintf(milliseconds, "%ld", V);
            printf("%s ms\n\r>", milliseconds);
            pwm_timer_set(0, V, 0, DO1, NOINVERT);
            while (pwm_timer(0)); // wait for timer expiration
        }
    } else {
        if (trigger_status()) { // if trigger switch pressed
            printf("manual trigger\n\r>");
            pwm_timer_set(0, 5000, 0, DO1, NOINVERT); // open valve
            while (TRIGGER == 0); // wait until switch released
            trigger_clear(); // clear status
            pwm_timer_reset(0); // close valve         
        }
    }
}
void dotting(int pulse) {
    if (AUTO_MAN == AUTO) {
        if (trigger_status()) { // if trigger switch pressed
            printf("auto trigger begin: %d ms pulse\n\r", pulse);
            trigger_clear(); // clear status
            while ((AUTO_MAN == AUTO) && (!trigger_status())) {
                V = (long) (adc_get())*3;
                pwm_timer_set(0, pulse, V, DO1, NOINVERT);
            }
            trigger_clear(); // clear status
            pwm_timer_set(0, 0, 0, DO1, NOINVERT);
            printf("auto trigger stop\n\r");
        }
    } else {
        if (trigger_status()) { // if trigger switch pressed
            printf("manual trigger\n\r>");
            trigger_clear(); // clear status
            pwm_timer_set(0, pulse, 0, DO1, NOINVERT);
            while (pwm_timer(0)); // wait for timer expiration         
        }
    }
}

int main(void) {
    
/* initialize the system ******************************************************/
    proc_init(); // initialize processor
    set_msg_buffers(); // assign message buffers to mailboxes
    tb_initialize(PROC_TICKS_PER_COUNT); // initialize timebase
    __C30_UART = UI_UART; // select uart for stdio
    ser_init(BAUD_9600, msg_flg_ptr(UI), get_msg_buffer(UI)); // intialize uart
    tb_delay_ms(1000);
    adc_init();
    start_screen(); // show start screen
    printf(">"); // UI prompt
    // set timers
    pwm_timer_set(1, 50, 950, LED1, NOINVERT);
    pwm_timer_set(2, 100, 9900, CLOCKOUT, NOINVERT);
    
/******************************************************************************/

/***** execute main loop ******************************************************/
    while (1) {
        process_msg(get_token_ptrs(), UI);
        switch (read_dipsw()) {
            case 0:
                oneshot(5); // short duration pulses
                break;
            case 1:
                dotting(10); // 10ms pulse, repeating
                break;
            case 2:
                dotting(50); // 50ms pulse, repeating
                break;
            case 3:
                dotting(100); // 100ms pulse, repeating
                break;
            case 4:
                dotting(150); // 150ms pulse, repeating
                break;
            case 5:
                dotting(200); // 200ms pulse, repeating
                break;
            case 6:
                dotting(250); // 250ms pulse, repeating
                break;
            case 7:
                dotting(300); // 300ms pulse, repeating
                break;
            case 8:
                dotting(350); // 350ms pulse, repeating
                break;
            case 9:
                dotting(400); // 400ms pulse, repeating
                break;
            case 0xa:
                dotting(450); // 450ms pulse, repeating
                break;
            case 0xb:
                dotting(500); // 500ms pulse, repeating
                break;
            case 0xc:
                dotting(550); // 550ms pulse, repeating
                break;
            case 0xd:
                dotting(600); // 600ms pulse, repeating
                break;
//            case 0xe: // calibrate time
//                TRISBbits.TRISB15 = 0; // pin 18, reference clock output
//                REFOCON = 0x8000;
//                while (1);
//                break;
            case 0xf: 
                oneshot(10); // long duration pulses
                break;
        }
    }
/******************************************************************************/
    return 0;
}

