

#include "msg_processor.h"

char* token_ptrs[MAX_TOKENS];
const char null_string = '\0'; // used to set token_ptrs to point to null

char** get_token_ptrs(void) {
    return token_ptrs;
}

void clear_token_ptrs(void) {
    int i;
    for (i = 0; i < MAX_TOKENS; i++) {
        token_ptrs[i] = (char*) &null_string;
    }
}

void add_token_ptr(int position, char* pointer) {
    token_ptrs[position] = pointer;
}
struct mailslot {
    int flg;
    char* mailbox;
};

volatile static struct mailslot mailslots[NUM_MAILSLOTS];

typedef int (*command_ptr)(char**); // function pointer for commands

struct keyword_table{
    char keyword[KEYWORD_MAX_LEN]; // keyword for function to be executed
    int match_len; // minimum # characters required for match
    command_ptr command; // function pointer
};

const struct keyword_table keywords[] = {
    // keyword, match_len, function
    {"RESET", 3, soft_reset},
    {"SHUTDOWN", 2, sleep},
    {"LED TEST", 3, led_test},
    {"OC TEST", 2, dout_test},
    {"SWITCH", 3, switch_test}, 
    {"TIME", 3, vtim},
    {"DIPSW", 3, dipsw},
    
    {"\0", 0, 0}
};

void preprocess(char* ptr){
    int has_sf = 0;
    char insert;
    char temp;
    if(*ptr == START_FLAG) has_sf = 1;
    insert = START_FLAG;
    while((insert != 0) && (insert != END_FLAG)) {
        if(!has_sf) { //no start flag, so insert one and push out rest of chars
            temp = *ptr;
            *ptr = insert;
            insert = temp;
        }
        else { // has a start flag, so search for end flag
            insert = *ptr;
        }
        ptr++;
    }
    if(insert == 0) { // no end flag found, so add one
        *ptr = END_FLAG; // overwrites string terminator
        ptr++;
        *ptr = 0; // add new string terminator
    }
}

int process_msg(char* fields[], exchanges mailslot) {
    int err = 0;
    char* inptr;
    int i;
    int found = 0;
    int fl;
    if (get_msg_flag(mailslot)) {
        clr_msg_flag(mailslot);
        for (i = 0; i < MAX_TOKENS; i++) { // point all field ptrs to null
            fields[i] = (char*) &null_string;
        }
        if (mailslot == UI) preprocess(get_msg_buffer(mailslot));
        inptr = parse_frm(get_msg_buffer(mailslot), &fields[0], START_FLAG, END_FLAG, DELIMITER, 1);
        if (get_msg_buffer(mailslot) == inptr) {
            err = 1; // returned same pointer it was sent, means failed parse
        } else { // lookup keyword
            i = 0;
            while ((!found) && (*keywords[i].keyword != 0)) {
                fl = str_len(fields[0]);
                if ((fl >= keywords[i].match_len) &&
                        (fl <= str_len((char*) keywords[i].keyword))) {
                    if (str_match(fields[0], (char*) keywords[i].keyword, fl, 1)) {
                        found = 1;
                        err = (keywords[i].command)(&fields[0]); // execute the function
                    }
                }
                i++;
            }
        }
        if ((!found) && (!err)) err = 2;
        show_err(err);
        if (mailslot == UI) ser_putchar('>');
    }
    return err;
}

//void send_command(char* fields[], int num_tokens, char* s_dest_mod) {
//}

char* parse_frm(char *src, char* vect[], char sf, char ef, char dlm, int trim) {
    char *ptr;
    ptr = src; // ptr is used as a pointer to the current position in input
    int i = 0; // iterates thru the result vector
    int seek_token = 1; // flag to indicate whether seeking token start or delim
    while ((*ptr != sf) && (*ptr != ef) && (*ptr != 0)) ptr++; // scan for start
    if ((*ptr == ef) || (*ptr == 0)) return src; // no start flag found, so give up
    ptr++; // skip over the start flag
    while (*ptr != ef) {
        while ((trim == 1) && (*ptr == ' ')) ptr++; // skip leading spaces
        if (*ptr == 0) return src; // premature end of string
        if (*ptr == dlm) {
            if (seek_token) { // found delim while seeking token, token is empty
                *ptr = '\0'; // insert null to indicate end of string
                vect[i++] = ptr; // save pointer to result vector
            } else {
                *ptr = '\0'; // found delim, so insert null, seek next token
                seek_token = 1;
            }
            } else if (seek_token) { // found start of token, save ptr to result vector
            vect[i++] = ptr;
            seek_token = 0; // token already found, scan til end of token
        }
        ptr++;
    }
    // at this point, the end flag is found, replace it and return pointer
    *ptr = '\0';
    return ptr;
}

char* assign_msg_buffer(int exchange, char* msg_buffer) {
    mailslots[exchange].mailbox = msg_buffer;
    return mailslots[exchange].mailbox;
}

char* get_msg_buffer(int exchange) {
    return mailslots[exchange].mailbox;
}

int* msg_flg_ptr(int exchange) {
    return (int*)&mailslots[exchange].flg;
}

void set_msg_flag(int exchange) {
    mailslots[exchange].flg = 1;
}

void clr_msg_flag(int exchange) {
    mailslots[exchange].flg = 0;
}

int get_msg_flag(int exchange) {
    return mailslots[exchange].flg;
}

