
#include "setup.h"

// Configuration bits

// FBS
#pragma config BSS = OFF    // Boot segment Protect->No boot program flash segment
#pragma config BWRP = OFF    // Boot Segment Write Protect->Disabled

// FGS
#pragma config GSS0 = OFF    // General Segment Code Protect->No Protection
#pragma config GWRP = OFF    // General Segment Write Protect->General segment may be written

// FOSCSEL
#pragma config LPRCSEL = HP    // LPRC Oscillator Power and Accuracy->High Power, High Accuracy Mode
#pragma config IESO = ON    // Internal External Switch Over bit->Internal External Switchover mode enabled (Two-speed Start-up enabled)

#ifdef EXT_OSC
#pragma config FNOSC = PRI    // Oscillator Select->Primary Oscillator (XT, HS, EC)
#endif // EXT_OSC

#ifdef INT_OSC
#ifdef CK32MHZ
#pragma config FNOSC = FRCPLL    // Oscillator Select->Fast internal oscillator w/ PLL
#endif // CK32MHZ
#ifdef CK16MHZ
#pragma config FNOSC = FRCPLL    // Oscillator Select->Fast RC oscillator w/PLL
#endif // CK8MHZ
#ifdef CK8MHZ
#pragma config FNOSC = FRC    // Oscillator Select->Fast RC oscillator
#endif // CK8MHZ
#ifdef CK4MHZ
#pragma config FNOSC = FRCDIV    // Oscillator Select->Fast RC oscillator w/ postscaler
#endif // CK8MHZ
#endif // INT_OSC

#pragma config SOSCSRC = DIG    // SOSC Source Type->Analog Mode for use with crystal *** IF NOT SET TO DIG, WILL PREVENT USE AS DIG IO !!! ***

// FOSC
#pragma config POSCFREQ = HS    // Primary Oscillator Frequency Range Configuration bits->Primary oscillator/external clock input frequency greater than 8MHz

#ifdef EXT_OSC
#pragma config POSCMOD = EC    // Primary Oscillator Configuration bits->External clock mode selected
#endif // EXT_OSC

#ifdef INT_OSC
#pragma config POSCMOD = NONE    // Primary Oscillator Configuration bits->High speed osc mode selected
#endif // INT_OSC

#pragma config SOSCSEL = SOSCHP    // SOSC Power Selection Configuration bits->Secondary Oscillator configured for high-power operation
#pragma config OSCIOFNC = OFF    // CLKO Enable Configuration bit->CLKO output disabled
#pragma config FCKSM = CSDCMD    // Clock Switching and Monitor Selection->Both Clock Switching and Fail-safe Clock Monitor are disabled

// FWDT
#pragma config WDTPS = PS32768    // Watchdog Timer Postscale Select bits->1:32768
#pragma config FWPSA = PR128    // WDT Prescaler bit->WDT prescaler ratio of 1:128
#pragma config WINDIS = OFF    // Windowed Watchdog Timer Disable bit->Standard WDT selected(windowed WDT disabled)
#pragma config FWDTEN = OFF    // Watchdog Timer Enable bits->WDT disabled in hardware; SWDTEN bit disabled

// FPOR
#pragma config I2C1SEL = PRI    // Alternate I2C1 Pin Mapping bit->Use Default SCL1/SDA1 Pins For I2C1
#pragma config BOREN = BOR3    // Brown-out Reset Enable bits->Brown-out Reset enabled in hardware, SBOREN bit disabled
#pragma config LVRCFG = OFF    // ->Low Voltage regulator is not available
#pragma config MCLRE = ON    // MCLR Pin Enable bit->RA5 input pin disabled,MCLR pin enabled
#pragma config BORV = V20    // Brown-out Reset Voltage bits->Brown-out Reset set to lowest voltage (1.8V)
#pragma config PWRTEN = ON    // Power-up Timer Enable bit->PWRT enabled

// FICD
#pragma config ICS = PGx2    // ICD Pin Placement Select bits->EMUC/EMUD share PGC1/PGD1

// FDS
#pragma config DSWDTPS = DSWDTPSF    // Deep Sleep Watchdog Timer Postscale Select bits->1:2,147,483,648 (25.7 Days)
#pragma config DSWDTOSC = LPRC    // DSWDT Reference Clock Select bit->DSWDT uses Low Power RC Oscillator (LPRC)
#pragma config DSBOREN = ON    // Deep Sleep Zero-Power BOR Enable bit->Deep Sleep BOR enabled in Deep Sleep
#pragma config DSWDTEN = OFF    // Deep Sleep Watchdog Timer Enable bit->DSWDT disabled

void proc_init(void) {
    /****************************************************************************
     * Setting the GPIO of PORTA
     ***************************************************************************/
    LATA = 0x00;
    TRISA = 0x0008;
    /****************************************************************************
     * Setting the GPIO of PORTB
     ***************************************************************************/
    LATB = 0x00;
    ODCB = 0x3300; // configure B13, B12, B9, B8 as open drain
    ODCA = 0x0002; // configure RA2 as open drain

    /****************************************************************************
     * Setting the Analog/Digital Configuration SFR
     ***************************************************************************/
    ANSA = 0x0000; // disable analog inputs
    ANSB = 0x0001; // disable analog inputs, except AN2

    TRISBbits.TRISB1 = 0; // pin 5, Open Collector driver
    TRISBbits.TRISB2 = 1; // pin 6, RX, input 
    TRISAbits.TRISA3 = 0; // pin 8, LED1, output
    TRISBbits.TRISB4 = 1; // pin 9, AUTO/MAN switch, input
    TRISBbits.TRISB7 = 0; // pin 11, TX, output
    TRISBbits.TRISB8 = 0; // pin 12, DIP switch output, open drain
    TRISBbits.TRISB9 = 0; // pin 13, DIP switch output, open drain
    TRISBbits.TRISB12 = 0; // pin 15, DIP switch output, open drain
#if (MODEL==0)
    TRISBbits.TRISB13 = 0; // pin 16, DIP switch output, open drain
    TRISAbits.TRISA4 = 1; // pin 10, input
#endif // MODEL==0
#if (MODEL==1)
    TRISAbits.TRISA4 = 0; // pin 10, DIP switch output, open drain
    TRISBbits.TRISB13 = 1; // pin 16, DIP switch input
#endif // MODEL==0
    TRISBbits.TRISB14 = 1; // pin 17, DIP switch input
    TRISBbits.TRISB15 = 1; // pin 18, trigger switch input
    TRISAbits.TRISA2 = 0; // pin 7, output, clock

#ifdef INT_OSC
#ifdef CK32MHZ
    CLKDIV = 0x3000; // divide by one
#endif // CK32MHZ   
#ifdef CK16MHZ
    CLKDIV = 0x3100; // divide by 2
#endif // CK16MHZ   
#ifdef CK8MHZ
    CLKDIV = 0x3000; // divide by one
#endif // CK8MHZ   
#ifdef CK4MHZ
    CLKDIV = 0x3100; // divide by 2
#endif // CK4MHZ   
#endif
}

// declare message buffers
static char msg_buffer_ui[UI_BFR_SIZE]; // user interface input

void start_screen() {
    printf("\033[2J"); // clear screen
    printf("\033[f"); // home position
    printf("K8VFO Timer/Controller\n\n\r");
    printf("System started...\n\n\r");
}

void set_msg_buffers(void) {
    // assign message buffers to mailboxes
    assign_msg_buffer(UI, msg_buffer_ui); // user interface input
}

