/* 
 * File:   sys_config.h
 * Author: WHalphen
 * Created on October 20,2016
 * 
 * System configuration values are set here
 */

#ifndef SYS_CONFIG_H
#define	SYS_CONFIG_H

// system configuration options ************************************************
#define CLOCK_SOURCE 1 // enter 1 for INTERNAL or 0 for EXTERNAL
#define CLOCK 4 // enter oscillator frequency in MHz: 32|16|8|4
#define p24fV32KA301 // CPU
#define DEBUG_MODE 1 // enter 1 for YES or 0 for NO 
#define MODEL 1 // enter 0 for 5 pin DIP connector, 1 for 6 pin DIP connector
//******************************************************************************


#ifdef p24fV32KA301
    #include <p24fV32KA301.h>
#endif // p24fV32KA301

#if (CLOCK_SOURCE==1)
    #define INT_OSC // Internal oscillator
#endif
#if (CLOCK_SOURCE==0)
    #define EXT_OSC // External oscillator
#endif

#define UI_UART 1
#define UI_UART1

// clock and rate definitions
#if (CLOCK==32)
    #define CK32MHZ
    // baud rate definitions for 32MHz clock
    #define BAUD_1200 832
    #define BAUD_9600 103
    #define BAUD_19200 51
    #define BAUD_115200 8
    #define BRG 37 // I2C clock 400 kHz
    #define PROC_TICKS_PER_COUNT 1600 // timebase counter value
#endif
#if (CLOCK==16)
    #define CK16MHZ
    // baud rate definitions for 16MHz clock
    #define BAUD_1200 416
    #define BAUD_9600 51
    #define BAUD_19200 25
    #define BRG 18 // I2C clock 400 kHz
    #define PROC_TICKS_PER_COUNT 800 // timebase counter value
#endif
#if (CLOCK==8)
    #define CK8MHZ
    // baud rate definitions for 8MHz clock
    #define BAUD_1200 207
    #define BAUD_9600 25
    #define BAUD_19200 12
    #define BRG 9 // I2C clock 400 kHz
    #define PROC_TICKS_PER_COUNT 400 // timebase counter value
#endif
#if (CLOCK==4)
    #define CK4MHZ
    // baud rate definitions for 4MHz clock
    #define BAUD_1200 103
    #define BAUD_9600 12
    #define BRG 19 // I2C clock 100 kHz
    #define PROC_TICKS_PER_COUNT 200 // timebase counter value
#endif
#define CLOCK_ADJUST_INCR (PROC_TICKS_PER_COUNT / 200)

#if (DEBUG_MODE==1)
    #define DEBUG
#endif

// link PWM timer to physical pin
#define DOUT1 LATBbits.LATB1 //digital output 1
#define DOUT2 LATAbits.LATA3 //digital output 2
#define DOUT3 LATAbits.LATA2 //digital output 3
#define DOUT4 LATBbits.LATB14 //digital output 4
#define DOUT5 LATAbits.LATA2 //digital output 5
#define LED1 2 //digital output 2 is LED1
#define DO1 1 // digital output 1 is DO1, open collector driver

#if (MODEL==0)
    #define DIPSW1 LATBbits.LATB8 // O.D. output to dip switch 1
    #define DIPSW2 LATBbits.LATB9 // O.D. output to dip switch 2
    #define DIPSW3 LATBbits.LATB12 // O.D. output to dip switch 3
    #define DIPSW4 LATBbits.LATB13 // O.D. output to dip switch 4
    #define SENSE PORTBbits.RB14 // senses input from dip switches
#endif //MODEL 0

#if (MODEL==1)
    #define DIPSW1 LATAbits.LATA4 // O.D. output to dip switch 1
    #define DIPSW2 LATBbits.LATB8 // O.D. output to dip switch 2
    #define DIPSW3 LATBbits.LATB9 // O.D. output to dip switch 3
    #define DIPSW4 LATBbits.LATB12 // O.D. output to dip switch 4
//    #define DIPSW4 LATBbits.LATB13 // senses input from dip switches
    #define SENSE PORTBbits.RB14 // senses input from dip switches
#endif //MODEL 1

#define CLOCKOUT 5


#define INVERT 1 // invert output of timer
#define NOINVERT 0 // non-inverted output of timer
#define NUM_TIMERS 3 // pwm timers
//#define NUM_CYCLE_TIMERS 4 // cycle timers

// inter process messaging
#define NUM_MAILSLOTS 1
#define MAX_TOKENS 15
typedef enum {UI} exchanges;
#define UI_BFR_SIZE 80


#define MAX_FIELDS 21 // max number of fields in a message

#define TRIGGER PORTBbits.RB15 // trigger switch input
#define AUTO_MAN PORTBbits.RB4 // auto/manual switch input
#define AUTO 0
#define MANUAL 1

// digital output modes
typedef enum {PULSE, PWM, DO_ON, DO_OFF} dout_modes;
    
#define START_FLAG '$'
#define END_FLAG '*'
#define DELIMITER ','

#define KEYWORD_MAX_LEN 8

#endif	/* SYS_CONFIG_H */

